/*
  ResHiggs code based on

    M. Bonvini, L. Rottoli
    arXiv:1412.3791

  please cite it if you use it
*/

#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <getopt.h> 

#include "../ggHiggs/src/ggHiggs.hh"
#include "./src/resummationSCET.hh"

#include "LHAPDF/LHAPDF.h"


using namespace std;



/*
  If you like, you can customize your alpha_s routine.
  However, consistency with PDF sets suggests the use of the LHAPDF alphas routine.
*/
double alphas(double mu) {
  return LHAPDF::alphasPDF(mu);
  //return alphas_ihixs(mu, order, LHAPDF::alphasPDF(mZ), mZ); // taken from ihixs code
}





void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -t  --finite-mt         use finite top mass (default: infinite-mtop approximation)" << endl
       << " -p  --pdf-set <set>     specify PDFs being used (default \"NNPDF23_nnlo_as_0118\")" << endl
       << " -m  --mass <mass>       set Higgs mass [GeV]" << endl
       << " -E  --Ecm <energy>      set collider energy [GeV]" << endl
       << " -f  --fact <murat>      set muF/mH  ratio" << endl
       << " -r  --ren <murat>       set muR/muF ratio" << endl
       << " -H  --hard <murat>      set muH/mH  ratio" << endl
       << " -c  --collinear         include collinear contributions" << endl
       << " -2  --pi2               perform pi^2 resummation" << endl
       << endl;
  exit(0);
  return;
}




double muHrat = 1;
double muRrat = 1;
double muFrat = 1;
bool pi2 = false;
bool collinear = false;

const string sorder[4] = {"LO", "NLO", "NNLO", "NNNLO"};
void read_arguments(int argc, char* argv[], double &mH, double &sqrts, string &set) {
  const char* const short_options = "htp:f:r:h:m:E:c2";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "finite-mt", 0, NULL, 't' },
					 { "pdf-set", 1, NULL, 'p' },
					 { "fact", 1, NULL, 'f' },
					 { "ren", 1, NULL, 'r' },
					 { "hard", 1, NULL, 'H' },
					 { "mass", 1, NULL, 'm' },
					 { "Ecm", 1, NULL, 'E' },
					 { "collinear", 0, NULL, 'c' },
					 { "pi2", 0, NULL, '2' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 't':
      finite_mt = true;
      break;
    case 'p':
      sset << optarg;
      set = sset.str();
      break;
    case 'f':
      muFrat = strtod(optarg, NULL);
      break;
    case 'r':
      muRrat = strtod(optarg, NULL);
      break;
    case 'H':
      muHrat = strtod(optarg, NULL);
      break;
    case 'm':
      mH = strtod(optarg, NULL);
      break;
    case 'E':
      sqrts = strtod(optarg, NULL);
      break;
    case 'c':
      collinear = true;
      break;
    case '2':
      pi2 = true;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  // messages
  //
  if(muRrat!=1)
    cout << endl << "\033[1m  WARNING:\033[0m "
	 << "\033[0;31m In SCET there is no muR, so setting muR to a value different from muF makes no sense at resummed level.\033[0m"
	 << endl << endl;
  //
  if(!finite_mt) cout << ">>> Infinite m_t limit <<<" << endl;
  else           cout << ">>> Finite m_t !! <<<" << endl;
  //
  cout << ">>> PDFset: " << set << endl;
  //
  _muF_mu_ratio  = muFrat;
  _muR_muF_ratio = muRrat;
  cout << "Renormalization scale muR = " << muRrat*muFrat << " * mH" << endl;
  cout << "Factorization   scale muF = " << muFrat << " * mH" << endl;
  return;
}







int main (int argc, char* argv[]) {


  /*************  input parameters  **************/

  // pole masses (GeV)
  mtop = 172.5;
  mbot = 4.75;     // negative mass means no bottom running in the loop (consistent with small-x coefficients)
  mcha = sqrt(2.); // negative mass means no charm  running in the loop (consistent with small-x coefficients)

  // relative precision for integrals
  INTEGRAL_PRECISION = 5.e-3;

  // scheme at small x (set to "false" for Q0MSbar scheme)
  MSbar = true;

  // DGLAP (collinear) improvement at the exponent (natural)
  // or at the level of coefficient function (unnatural, but easier and faster)
  APimprExponent = true;
  // not needed if ggHiggs is used just for fixed order up to NNLO

  /***********************************************/



  // default values (modified by runtime arguments)
  double mH = 125;
  double sqrts = 8000;
  string PDFset = "NNPDF23_nnlo_as_0118.LHgrid";


  // read runtime arguments
  read_arguments(argc, argv, mH, sqrts, PDFset);




  // Start time counter
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);




  // init
  double tau = mH*mH/sqrts/sqrts;
  sxc = new SmallxCoeffs("../ggHiggs/src/MBDFV.dat");
  init_consts(mH);
  //
  cout << "Higgs mass:       " << mH << " GeV" << endl
       << "Collider energy:  " << sqrts << " GeV" << endl
       << "tau = mH^2/s =    " << tau << endl;
  //





  /******* RUN EXAMPLE *******/

  // set to false to show intermediate K-factor results
  quiet = true;
  //
  // init PDFs
  int subset = 0;
  pdf_init(PDFset, subset);
  //
  //
  // Note also that the same PDF set has to be used at every order.
  double xsLO=1, xsNLO=0, xsNNLO=0;
  //
  double LOpref = xs_LO_prefactor(mH);
  double muF = mH  * muFrat;
  double muR = muF * muRrat;
  double alpha_s = alphas(muR);
  // results
  double rexact[2];

  // LO
  xsLO = tau*Lum(tau,muF)*LOpref*pow(alpha_s,2);

  // NLO
  order = 1;
  __all_channels__ = true;
  compute(mH, sqrts, xsLO, 0, 0, &rexact[0]);
  xsNLO = xsLO + rexact[0];

  // NNLO
  order = 2;
  __all_channels__ = true;
  compute(mH, sqrts, xsLO, xsNLO, 0, &rexact[1]);
  xsNNLO = xsNLO + rexact[1];



  // *** Resummation part *** //
  //
  using namespace resumm;
  SCETResummation res(ggH, LHC, PDFset, subset, SILENT);
  //
  res.SetMtop(mtop);
  res.SetMbottom(mbot);
  res.SetMcharm(mcha);
  res.UseLargeMtopEFT(!finite_mt);
  res.SetLogCounting(primed);
  if(collinear) res.SetAPimprovement(AP1);
  res.UsePi2Resummation(pi2);
  //

  // you can specify the value of muS yourself
  //res.Setmus(...);
  // you might want to use res.GetmusI(mH,sqrts) and res.GetmusII(mH,sqrts), for computing scale uncertainty for instance.


  // compute resummed contribution
  double xsNNNLL = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muHrat, NNLO);
  //
  // end resummation part //




  cout << endl
       << "The following cross-sections (in pb) have been computed with the same PDF set (" << PDFset << ") at each order."
       << endl << endl;
  cout << "LO:         "  << xsLO << endl;
  cout << "NLO:        " << xsNLO << endl;
  cout << "NNLO:       " << xsNNLO << endl;
  cout << "NNLO+N3LL': " << xsNNLO+xsNNNLL << endl;
  //





  // Stop time counter
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << endl << "Total time: " << t << "s" << endl;


  return 0;

}
